#!/bin/sh
#
#	Compile a single executable in a subdirectory from two .c
#	files.  The executable prints a string supplied (through
#	a #define) by a STRING= argument to the invoking cons
#	command line.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [\$ARG{}]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir src
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >Construct
$print = $ARG{STRING};
$env = new cons (
	CFLAGS	=> "-DFOO=\\\"$print\\\"",
);
Export qw( env );
Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/Conscript
Import qw( env );
Program $env 'foo', qw (
	a.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/a.c
a()
{
	printf("In function a().\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/main.c
main()
{
	a();
	printf("%s!\n", FOO);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . STRING='antidisestablishmentarianism' > cons.1.out 2>&1
if test $? -ne 0 ; then fail ; fi
src/foo > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
In function a().
antidisestablishmentarianism!
_EOF_
if test $? -ne 0 ; then fail ; fi

#
pass
