#!/bin/sh
#
#	Create a work subdirectory and a repository subdirectory.
#	The work subdirectory contains four .c files and a Construct
#	file with Repository pointing to a non-existant repository.
#	Invoke cons in the work subdirectory.  Make sure the
#	generated executable was built successfully from the work
#	subdirectory .c files.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Repository, no repository directory]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr, input: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then exit 2 ; fi

#
cat <<_EOF_ >work/Construct		# NO SINGLE QUOTES TO EXPAND $work
\$env = new cons;
Repository '$work/no_repository_here';
Program \$env 'foo', qw (
	aaa.c
	bbb.c
	ccc.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/aaa.c
aaa()
{
	printf("work/aaa.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/bbb.c
bbb()
{
	printf("work/bbb.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/ccc.c
ccc()
{
	printf("work/ccc.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/main.c
main()
{
	aaa();
	bbb();
	ccc();
	printf("work/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - cons.1.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
work/aaa.c
work/bbb.c
work/ccc.c
work/main.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd work && $prog .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
