#!/bin/sh
#
#	Create a repository directory and a work directory, with
#	a src directory under each.  The repository/src directory
#	contains three .c files and a Conscript file.  The work
#	Construct file Builds the Conscript file and Repository
#	points to the repository.  Invoke cons in the work
#	subdirectory.  Check that it built the executable in the
#	src subdirectory, correctly picking up the files from the
#	repository.  Build again; check that nothing is rebuilt.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Repository Build]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr, input: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/src
if test $? -ne 0 ; then no_result ; fi

mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src
if test $? -ne 0 ; then no_result ; fi

#
cat <<_EOF_ >work/Construct		# NO SINGLE QUOTES TO EXPAND $work
Repository '$work/repository';
Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/Conscript
$env = new cons;
Program $env 'foo', qw (
	a.c
	b.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/a.c
a()
{
	printf("repository/src/a.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/b.c
b()
{
	printf("repository/src/b.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/main.c
main()
{
	a();
	b();
	printf("repository/src/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail ; fi
work/src/foo > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
repository/src/a.c
repository/src/b.c
repository/src/main.c
_EOF_
if test $? -ne 0 ; then fail ; fi

(set +x; cd work && $prog .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
