#!/bin/sh
#
#	Compile a single executable in a subdirectory from three
#	.c files, two in a repository and one in a separate
#	subdirectory.   The .c file in the separate subdirectory
#	is specified with an absolute (initial '/') path name.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [absolute (/) path name, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir xxx
if test $? -ne 0 ; then no_result ; fi

#
cat <<_EOF_ >repository/Construct	# NO SINGLE QUOTES TO EXPAND $work
\$env = new cons;
Program \$env 'foo', qw (
	a.c
	$work/xxx/a.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/a.c
src_a()
{
	printf("repository/a.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >xxx/a.c
xxx_a()
{
	printf("xxx/a.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/main.c
main()
{
	src_a();
	xxx_a();
	printf("repository/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
repository/a.c
xxx/a.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail ; fi

#
pass
