#!/bin/sh
#
#	Compile a single executable from a single repository .c
#	file which includes a single .h file from one of two
#	repository subdirectories.  Both subdirectories have the
#	same-named .h file.  Compile the executable twice, once
#	with the one subdirectory first in CPPPATH, then with the
#	other subdirectory first.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [CPPPATH, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/zark
if test $? -ne 0 ; then no_result ; fi

mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/include
if test $? -ne 0 ; then no_result ; fi
mkdir repository/zark
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >repository/foo.c
#include <foo.h>
main()
{
	printf(STRING);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

cat <<'_EOF_' >repository/include/foo.h
#define	STRING	"repository/include/foo.h\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/zark/foo.h
#define	STRING	"repository/zark/foo.h\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/zark/foo.h
#define	STRING	"work/zark/foo.h\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi

cat <<'_EOF_' >repository/Construct
$env = new cons (
	CPPPATH => 'include:zark',
);
Program $env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
repository/include/foo.h
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

cat <<'_EOF_' >work/Construct
$env = new cons (
	CPPPATH => 'zark:include',
);
Program $env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 2 ; fi
diff - foo.out <<'_EOF_'
work/zark/foo.h
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
