#!/bin/sh
#
#	Make an installation subdirectory in a repository and a
#	work subdirectory.  The repository Construct file specifies
#	installation of a shell script into the install subdirectory.
#	Build in the repository.  Build in the work subdirectory;
#	everything should be up-to-date.  Create a work copy of
#	the Construct file that specifies that the installed file
#	should be Local; build; see that the script was correctly
#	installed in the work/install subdirectory from the repository
#	copy.  Create a work copy of the script; build; see that
#	the work copy was installed.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Local]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/install
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/install
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >repository/Construct
$env = new cons;
Install $env 'install', 'foo.sh';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/foo.sh
#!/bin/sh
echo "repository/foo.sh called with \$0 = '$0'"
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository && $prog .) >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
sh repository/install/foo.sh > foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<'_EOF_'
repository/foo.sh called with $0 = 'repository/install/foo.sh'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
if test -e work/install/foo.sh ; then fail 2 ; fi

cat <<'_EOF_' >work/Construct
$env = new cons;
Install $env 'install', 'foo.sh';
Local qw(install/foo.sh);
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 ; fi
sh work/install/foo.sh > foo.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.3.out <<'_EOF_'
repository/foo.sh called with $0 = 'work/install/foo.sh'
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

cat <<'_EOF_' >work/foo.sh
#!/bin/sh
echo "work/foo.sh called with \$0 = '$0'"
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.4.out 2>cons.4.err
if test $? -ne 0 ; then fail 4 ; fi
sh work/install/foo.sh > foo.out
if test $? -ne 0 ; then fail 4 ; fi
diff - foo.out <<'_EOF_'
work/foo.sh called with $0 = 'work/install/foo.sh'
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

#
pass
