#!/bin/sh
#
#	Create two executables from two .c files in the repository.
#	Build in a work subdirectory; everything is up-to-date.
#	Touch one of the executables in the repository, making its
#	time-stamp out-of-sync with the .consign file.  Create a
#	work Construct file that specifies Repository_Sig_Times_OK
#	0; build; everything should still be "up-to-date" despite
#	the mismatched time-stamps.  Create a work Construct file
#	that specifies Repository_Sig_Times_OK 1; build; check that
#	a work copy of the executable was built.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Repository_Sig_Times_OK]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >repository/Construct
$env = new cons;
Program $env 'aaa', 'aaa.c';
Program $env 'bbb', 'bbb.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/aaa.c
main()
{
	printf("repository/aaa.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/bbb.c
main()
{
	printf("repository/bbb.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository && $prog .) > cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
repository/aaa > aaa.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - aaa.1.out <<'_EOF_'
repository/aaa.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
repository/bbb > bbb.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - bbb.1.out <<'_EOF_'
repository/bbb.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
if test -e work/aaa ; then fail 2 ; fi
if test -e work/bbb ; then fail 2 ; fi

chmod u+w repository/aaa.o
if test $? -ne 0 ; then no_result ; fi
sleep 1			# make sure the time is newere
if test $? -ne 0 ; then no_result ; fi
touch repository/aaa
if test $? -ne 0 ; then no_result ; fi
chmod u-w repository/aaa.o
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/Construct
$env = new cons;
Program $env 'aaa', 'aaa.c';
Program $env 'bbb', 'bbb.c';
Repository_Sig_Times_OK 0;
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) > cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 ; fi
diff - cons.3.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
if test -x work/aaa ; then fail 3 ; fi
if test -x work/bbb ; then fail 3 ; fi

cat <<'_EOF_' >work/Construct
$env = new cons;
Program $env 'aaa', 'aaa.c';
Program $env 'bbb', 'bbb.c';
Repository_Sig_Times_OK 1;
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) > cons.4.out 2>cons.4.err
work/aaa > aaa.4.out
if test $? -ne 0 ; then fail 4 ; fi
diff - aaa.4.out <<'_EOF_'
repository/aaa.c
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
if test -x work/bbb ; then fail 4 ; fi

#
pass
